@@ -6,7 +6,7 @@
  * Descr.: boolean and Meta-constraint predicate management - C part       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: fd_bool_c.c,v 1.4 2001/01/23 11:38:19 diaz Exp $ */
+/* $Id: fd_bool_c.c,v 1.7 2002/03/19 18:24:33 diaz Exp $ */
 
 #define OBJ_INIT Fd_Bool_Initializer
 
@@ -83,6 +83,7 @@
 /*---------------------------------*
  * Global Variables                *
  *---------------------------------*/
+
 static WamWord bool_tbl[NB_OF_OP];
 static WamWord bool_xor;
 
@@ -166,20 +167,29 @@ Bool Fd_Lte_2(WamWord le_word, WamWord re_word);
 
 
 
-#define BOOL_CSTR_2(f,a1,a2)                                                \
-    do {                                                                    \
-     if (!Fd_Check_For_Bool_Var(a1)) return FALSE;                          \
-     if (!Fd_Check_For_Bool_Var(a2)) return FALSE;                          \
-     PRIM_CSTR_2(f,a1,a2);                                                  \
-    } while(0)
+#define BOOL_CSTR_2(f, a1, a2)                    	\
+  do							\
+    {							\
+      if (!Fd_Check_For_Bool_Var(a1))			\
+	return FALSE;					\
+      if (!Fd_Check_For_Bool_Var(a2))			\
+	return FALSE;					\
+      PRIM_CSTR_2(f, a1, a2);				\
+    }							\
+  while (0)
 
-#define BOOL_CSTR_3(f,a1,a2,a3)                                             \
-    do {                                                                    \
-     if (!Fd_Check_For_Bool_Var(a1)) return FALSE;                          \
-     if (!Fd_Check_For_Bool_Var(a2)) return FALSE;                          \
-                                                             /* a3 is OK */ \
-     PRIM_CSTR_3(f,a1,a2,a3);                                               \
-    } while(0)
+
+#define BOOL_CSTR_3(f, a1, a2, a3)                    	\
+  do							\
+    {							\
+      if (!Fd_Check_For_Bool_Var(a1))			\
+	return FALSE;					\
+      if (!Fd_Check_For_Bool_Var(a2))			\
+	return FALSE;					\
+                                      /* a3 is OK */	\
+      PRIM_CSTR_3(f, a1, a2, a3);			\
+    }							\
+  while (0)
 
 
 
@@ -815,6 +825,8 @@ Add_Fd_Variables(WamWord e_word)
 
   if (tag_mask == TAG_LST_MASK)
     {
+      adr = UnTag_LST(word);
+
       Add_Fd_Variables(Car(adr));
       Add_Fd_Variables(Cdr(adr));
     }
