@@ -6,7 +6,7 @@
  * Descr.: mathematical support                                            *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: math_supp.c,v 1.4 2001/01/23 11:38:19 diaz Exp $ */
+/* $Id: math_supp.c,v 1.6 2002/03/19 18:24:33 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdarg.h>
@@ -681,6 +681,8 @@ Normalize(WamWord e_word, int sign, Poly *p)
   int i;
   long n1, n2, n3;
 
+ terminal_rec:
+
   DEREF(e_word, word, tag_mask);
 
   if (tag_mask == TAG_FDV_MASK)
@@ -736,16 +738,26 @@ Normalize(WamWord e_word, int sign, Poly *p)
   switch (i)
     {
     case PLUS_1:
-      return Normalize(le_word, sign, p);
+      e_word = le_word;
+      goto terminal_rec;
 
     case PLUS_2:
-      return Normalize(le_word, sign, p) && Normalize(re_word, sign, p);
+      if (!Normalize(le_word, sign, p))
+	return FALSE;
+      e_word = re_word;
+      goto terminal_rec;
 
     case MINUS_2:
-      return Normalize(le_word, sign, p) && Normalize(re_word, -sign, p);
+      if (!Normalize(le_word, sign, p))
+	return FALSE;
+      e_word = re_word;
+      sign = -sign;
+      goto terminal_rec;
 
     case MINUS_1:
-      return Normalize(le_word, -sign, p);
+      e_word = le_word;
+      sign = -sign;
+      goto terminal_rec;
 
     case TIMES_2:
 #if 1				/* optimize frequent use: INT*VAR */
