@@ -6,7 +6,7 @@
  * Descr.: all bips (to force the linker)                                  * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: all_pl_bips.pl,v 1.5 2001/09/17 16:01:50 diaz Exp $ */
+/* $Id: all_pl_bips.pl,v 1.7 2002/04/09 08:26:39 diaz Exp $ */
 
 '$use_all_pl_bips' :-
 	'$use_control',
@@ -58,4 +58,6 @@
 	'$use_expand',
 	'$use_pretty',
 	'$use_random',
-	'$use_sockets'.
+	'$use_sockets',
+ 	'$use_src_rdr'.
+
