@@ -6,7 +6,7 @@
  * Descr.: byte-code support                                               *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.c,v 1.5 2001/01/23 11:38:19 diaz Exp $ */
+/* $Id: bc_supp.c,v 1.8 2002/05/07 17:45:45 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -216,19 +216,15 @@ static WamCont BC_Emulate_Clause(DynCInf *clause);
 
 static WamCont BC_Emulate_Byte_Code(BCWord *bc);
 
-static
-  void Prep_Debug_Call(int func, int arity, int caller_func,
-		       int caller_arity);
+static void Prep_Debug_Call(int func, int arity, int caller_func,
+			    int caller_arity);
 
 
 
-#define BC_EMULATE_ALT             X2462635F656D756C6174655F616C74
-
 #define BC_EMULATE_CONT            X2462635F656D756C6174655F636F6E74
 
 #define CALL_INTERNAL_WITH_CUT     X2463616C6C5F696E7465726E616C5F776974685F637574
 
-Prolog_Prototype(BC_EMULATE_ALT, 0);
 Prolog_Prototype(BC_EMULATE_CONT, 0);
 Prolog_Prototype(CALL_INTERNAL_WITH_CUT, 3);
 
@@ -256,7 +252,7 @@ Prolog_Prototype(CALL_INTERNAL_WITH_CUT, 3);
 
 #define Fit_In_24bits(n)           ((unsigned) (n) < (1<<24))
 
-#define Op_In_Tbl(str,op)    BC_Op(*p)=op; BC_Atm(*p)=Create_Atom(str); p++
+#define Op_In_Tbl(str, op)  BC_Op(*p)=op; BC_Atm(*p) = Create_Atom(str); p++
 
 #if WORD_SIZE == 32
 
@@ -754,7 +750,8 @@ BC_Call_Terminal_Pred_3(WamWord pred_word, WamWord call_info_word,
       return debug_call_code;
     }
 
-  if ((pred = Lookup_Pred(func, arity)) == NULL)
+  pred = Lookup_Pred(func, arity);
+  if (pred == NULL)
     {				/* case: fail/0 from '$call_from_debugger' */
       if (func != atom_fail || arity != 0)
 	{
