@@ -6,7 +6,7 @@
  * Descr.: byte-Code support - header file                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: bc_supp.h,v 1.3 2001/01/23 11:38:19 diaz Exp $ */
+/* $Id: bc_supp.h,v 1.5 2002/05/10 16:35:25 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -39,16 +39,18 @@
 
 #ifdef BC_SUPP_FILE
 
-unsigned *byte_code;
 int byte_len;
 
 #else
 
-extern unsigned *byte_code;
 extern int byte_len;
 
 #endif
 
+				/* defined as this to avoid to force the   */
+				/* inclusion of bc_supp.o if not needed    */
+				/* (dynam_supp.c uses byte_code)           */
+unsigned *byte_code;
 
 
 
