@@ -6,7 +6,7 @@
  * Descr.: character input-output management                               * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,25 +22,34 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: char_io.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: char_io.pl,v 1.7 2002/03/19 18:24:34 diaz Exp $ */
 
 :-	built_in.
 
 '$use_char_io'.
 
 
-get_code_no_echo(Code) :-
-	set_bip_name(get_code_no_echo, 1),
-	'$call_c_test'('Get_Code_No_Echo_1'(Code)).
+get_key(Code) :-
+	set_bip_name(get_key, 1),
+	'$call_c_test'('Get_Key_1'(Code)).
 
-get_code_no_echo(SorA, Code) :-
-	set_bip_name(get_code_no_echo, 2),
-	'$call_c_test'('Get_Code_No_Echo_2'(SorA, Code)).
+get_key(SorA, Code) :-
+	set_bip_name(get_key, 2),
+	'$call_c_test'('Get_Key_2'(SorA, Code)).
+
+
+get_key_no_echo(Code) :-
+	set_bip_name(get_key_no_echo, 1),
+	'$call_c_test'('Get_Key_No_Echo_1'(Code)).
+
+get_key_no_echo(SorA, Code) :-
+	set_bip_name(get_key_no_echo, 2),
+	'$call_c_test'('Get_Key_No_Echo_2'(SorA, Code)).
 
 
 
 
-          /* get predicates */
+          % get predicates
 
 get_char(Char) :-
 	set_bip_name(get_char, 1),
@@ -76,7 +85,7 @@ get_byte(SorA, Byte) :-
 
 
 
-          /* unget predicates */
+          % unget predicates
 
 unget_char(Char) :-
 	set_bip_name(unget_char, 1),
@@ -111,7 +120,7 @@ unget_byte(SorA, Byte) :-
 
 
 
-          /* peek predicates */
+          % peek predicates
 
 peek_char(Char) :-
 	set_bip_name(peek_char, 1),
@@ -146,7 +155,7 @@ peek_byte(SorA, Byte) :-
 
 
 
-          /* put predicates */
+          % put predicates
 
 put_char(Char) :-
 	set_bip_name(put_char, 1),
