@@ -6,7 +6,7 @@
  * Descr.: file consulting - C part                                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,27 +22,13 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: consult_c.c,v 1.4 2001/06/07 18:22:15 diaz Exp $ */
+/* $Id: consult_c.c,v 1.9 2002/04/19 15:43:35 diaz Exp $ */
 
 #include <sys/types.h>
 
 #include "engine_pl.h"
 #include "bips_pl.h"
-
-#if defined(M_ix86_cygwin) || defined(M_ix86_win32)
-#include <process.h>
-#endif
-
-#ifdef M_ix86_win32
-#include <io.h>
-#include <fcntl.h>
-#else
-#include <unistd.h>
-#include <sys/wait.h>
-#endif
-
-
-
+#include "linedit.h"
 
 /*---------------------------------*
  * Constants                       *
@@ -62,9 +48,6 @@
 
 #if 1
 
-
-
-
 /*-------------------------------------------------------------------------*
  * CONSULT_2                                                               *
  *                                                                         *
@@ -75,18 +58,33 @@ Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
   char *tmp_file = Rd_String_Check(tmp_file_word);
   char *pl_file = Rd_String_Check(pl_file_word);
   char *singl_warn = (Flag_Value(FLAG_SINGLETON_WARNING)) ? NULL
-       : "--no-singl-warn";
-  StmInf *pstm = stm_tbl + stm_top_level_output;
+    : "--no-singl-warn";
+  StmInf *pstm_o = stm_tbl[stm_top_level_output];
+  StmInf *pstm_i = stm_tbl[stm_top_level_input];
   int pid;
-  FILE *f_out;
+  FILE *f_out, *f_in;
+  FILE **pf_in;
+  long save;
+  unsigned char *p = NULL;
   int status, c;
+  int save_use_le_prompt;
   char *arg[] = { "pl2wam", "-w", "--compile-msg", "--no-redef-error",
-    "--pl-state", tmp_file, "-o", tmp_file, pl_file,
-    singl_warn, NULL
-  };
+		  "--pl-state", tmp_file, "-o", tmp_file, pl_file,
+		  singl_warn, NULL };
+
+  save = SYS_VAR_SAY_GETC;
+#ifndef NO_USE_PIPED_STDIN_FOR_CONSULT
+  SYS_VAR_SAY_GETC = 1;
+  pf_in = &f_in;
+#else
+  f_in = NULL;
+  pf_in = NULL;
+#endif
+  Write_Pl_State_File(tmp_file_word);
+  SYS_VAR_SAY_GETC = save;
 
   Flush_All_Streams();
-  pid = M_Spawn_Redirect(arg, 0, NULL, &f_out, &f_out);
+  pid = M_Spawn_Redirect(arg, 0, pf_in, &f_out, &f_out);
   Os_Test_Error(pid == -1);
   if (pid == -2)
     {
@@ -96,15 +94,50 @@ Consult_2(WamWord tmp_file_word, WamWord pl_file_word)
       return FALSE;
     }
 
+  save_use_le_prompt = use_le_prompt;
+  use_le_prompt = 0;
   for (;;)
     {
+#if 1
       c = fgetc(f_out);
-      if (c < 0)
+#else
+      char c0;
+      c = (read(fileno(f_out), &c0, 1) == 1) ? c0 : EOF;
+#endif
+      if (c == EOF)
 	break;
-
-      Stream_Putc(c, pstm);
+	  
+#ifndef NO_USE_PIPED_STDIN_FOR_CONSULT
+      if (c == CHAR_TO_EMIT_WHEN_CHAR)
+	{
+	  if (p == NULL)
+	    {
+	      c = Stream_Getc(pstm_i);
+	      if (c == EOF)
+		{
+		eof_reached:
+		  p = "end_of_file.\n";
+		  c = *p++;
+		}
+	    }
+	  else
+	    {
+	      if (*p == '\0')
+		goto eof_reached;
+	      else
+		c = *p++;
+	    }
+	  fputc(c, f_in);
+	  fflush(f_in);
+	  continue;
+	}
+#endif
+      Stream_Putc(c, pstm_o);
     }
+  use_le_prompt = save_use_le_prompt;
 
+  if (f_in)
+    fclose(f_in);
   fclose(f_out);
 
   status = M_Get_Status(pid);
