@@ -6,7 +6,7 @@
  * Descr.: expand term management - C part                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: expand_c.c,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: expand_c.c,v 1.8 2002/06/04 16:51:24 diaz Exp $ */
 
 #include <sys/types.h>
 
@@ -149,6 +149,9 @@ Dcg_Trans_Body_4(WamWord dcg_body_word, WamWord in_word, WamWord out_word,
   top = Local_Top;		/* use local stack for the stack */
   opt_term_unif = TRUE;
 
+  in_word = Globalize_If_In_Local(in_word); 
+  out_word = Globalize_If_In_Local(out_word);
+
   return Unify(body_word,
 	       Dcg_Body(dcg_body_word, FALSE, in_word, out_word, NULL));
 }
@@ -292,6 +295,7 @@ Dcg_Body_On_Stack(WamWord dcg_body_word, WamWord in_word, WamWord out_word)
   DEREF(dcg_body_word, word, tag_mask);
   if (tag_mask == TAG_REF_MASK)
     {
+      adr = UnTag_REF(word);
       func = atom_phrase;
       arity = 1;
       goto non_term;
