@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: inl_protos.h,v 1.8 2002/04/19 15:43:35 diaz Exp $ */
+/* $Id: inl_protos.h,v 1.9 2002/06/24 10:26:24 diaz Exp $ */
 
 /*---------------------------------*
  * Constants                       *
@@ -220,6 +220,10 @@ WamWord Fct_Sign(WamWord x) FC;
 
 
 
+WamWord Fct_Min(WamWord x, WamWord y) FC;
+
+WamWord Fct_Max(WamWord x, WamWord y) FC;
+
 WamWord Fct_Pow(WamWord x, WamWord y) FC;
 
 WamWord Fct_Sqrt(WamWord x) FC;
