@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: os_interf_c.c,v 1.10 2002/05/07 17:45:45 diaz Exp $ */
+/* $Id: os_interf_c.c,v 1.12 2002/09/27 14:20:58 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,6 +41,7 @@
 #include <direct.h>
 #include <io.h>
 #include <winsock.h>
+#include <fcntl.h>
 #else
 #define _XOPEN_SOURCE_EXTENDED	/* for alpha/OSF (usleep prototype) */
 #include <dirent.h>
@@ -1092,6 +1093,71 @@ Exec_5(WamWord cmd_word, WamWord stm_in_word, WamWord stm_out_word,
 
 
 
+/*-------------------------------------------------------------------------*
+ * CREATE_PIPE_2                                                           *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Create_Pipe_2(WamWord stm_in_word, WamWord stm_out_word)
+{
+  int p[2];
+  int stm;
+  FILE *f_in, *f_out;
+  int atom;
+
+#ifdef M_ix86_win32
+  Os_Test_Error(_pipe(p, 4096, O_TEXT));
+#else
+  Os_Test_Error(pipe(p));
+#endif
+
+  Os_Test_Error((f_in = fdopen(p[0], "rt")) == NULL);
+  sprintf(glob_buff, "pipe_stream_in");
+  atom = Create_Allocate_Atom(glob_buff);
+  stm = Add_Stream_For_Stdio_Desc(f_in, atom, STREAM_MODE_READ, TRUE);
+  stm_tbl[stm]->prop.eof_action = STREAM_EOF_ACTION_RESET;
+  Get_Integer(stm, stm_in_word);
+
+  Os_Test_Error((f_out = fdopen(p[1], "wt")) == NULL);
+  sprintf(glob_buff, "pipe_stream_out");
+  atom = Create_Allocate_Atom(glob_buff);
+  stm = Add_Stream_For_Stdio_Desc(f_out, atom, STREAM_MODE_WRITE, TRUE);
+  Get_Integer(stm, stm_out_word);
+
+  return TRUE;
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * FORK_PROLOG_1                                                           *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+Bool
+Fork_Prolog_1(WamWord pid_word)
+
+{
+#if defined(M_ix86_win32)
+
+  Pl_Err_Resource(Create_Atom("not implemented"));
+  return FALSE;
+
+#else
+
+  int pid;
+
+  pid = fork();
+  Os_Test_Error(pid == -1);
+
+  return Get_Integer(pid, pid_word);
+
+#endif
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * SELECT_5                                                                *
  *                                                                         *
