@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: pred.pl,v 1.6 2002/03/19 18:24:35 diaz Exp $ */
+/* $Id: pred.pl,v 1.8 2002/10/01 12:56:21 diaz Exp $ */
 
 :-	built_in.
 
@@ -34,12 +34,15 @@ current_predicate(PI) :-
 	'$current_predicate'(PI).
 
 '$current_predicate'(PI) :-
+	'$call_c_test'('Current_Predicate_2'(PI, 0)).
+
+
+'$current_predicate_bips'(PI) :-
 	'$call_c_test'('Current_Predicate_2'(PI, 1)).
 
 
-
 '$current_predicate_any'(PI) :-
-	'$call_c_test'('Current_Predicate_2'(PI, 0)).
+	'$call_c_test'('Current_Predicate_2'(PI, 2)).
 
 
 '$current_predicate_alt' :-         % used by C code to create a choice-point
@@ -50,7 +53,7 @@ current_predicate(PI) :-
 
 predicate_property(PI, Property) :-
 	set_bip_name(predicate_property, 2),
-	'$current_predicate'(PI),
+	'$current_predicate_bips'(PI),
 	'$predicate_property1'(PI, Property).
 
 
