@@ -6,7 +6,7 @@
  * Descr.: scanner support                                                 *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: scan_supp.c,v 1.3 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: scan_supp.c,v 1.6 2002/03/19 18:24:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -69,11 +69,12 @@ static void Scan_Number(StmInf *pstm, Bool integer_only);
 
 static void Scan_Quoted(StmInf *pstm);
 
-static int Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0);
+static int Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, 
+			    Bool no_escape);
 
 
 
-#define   Unget_Last_Char       Stream_Ungetc(c_orig,pstm)
+#define   Unget_Last_Char       Stream_Ungetc(c_orig, pstm)
 
 
 
@@ -329,7 +330,7 @@ Scan_Number(StmInf *pstm, Bool integer_only)
 
   if (c == '\'')		/* 0'<character> */
     {
-      c = Scan_Quoted_Char(pstm, TRUE, '\'');
+      c = Scan_Quoted_Char(pstm, TRUE, '\'', FALSE);
       if (c == -1)		/* <character> is ' */
 	{
 	  token.line = pstm->line_count + 1;
@@ -423,15 +424,32 @@ Scan_Quoted(StmInf *pstm)
   int c0;
   char *s;
   Bool convert = (c_orig != '\'');
+  Bool no_escape;
+  int i = 0;
+
+  if (c_type == QT)
+    {
+      token.type = TOKEN_NAME;
+      i = 0;
+    }
+  else if (c_type == DQ)
+    {
+      token.type = TOKEN_STRING;
+      i = Flag_Value(FLAG_DOUBLE_QUOTES);
+    }
+  else
+    {
+      token.type = TOKEN_BACK_QUOTED;
+      i = Flag_Value(FLAG_BACK_QUOTES);
+    }
 
-  token.type = (c_type == QT) ? TOKEN_NAME
-    : (c_type == DQ) ? TOKEN_STRING : TOKEN_BACK_QUOTED;
   s = token.name;
   c0 = c;
+  no_escape = i >> FLAG_NO_ESCAPE_BIT;
 
   for (;;)
     {
-      c = Scan_Quoted_Char(pstm, convert, c0);
+      c = Scan_Quoted_Char(pstm, convert, c0, no_escape);
       if (c == -1)
 	{
 	  *s = '\0';
@@ -484,7 +502,7 @@ Scan_Quoted(StmInf *pstm)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 static int
-Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0)
+Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0, Bool no_escape)
 {
   int radix;
   char *p, *f;
@@ -503,14 +521,14 @@ Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0)
   if (c == EOF || c == '\n')
     return -2;
 
-  if (c != '\\')
+  if (c != '\\' || no_escape)
     return c;
   /* escape sequence */
   Read_Next_Char(pstm, convert);
   if (c == '\n')		/* \ followed by \n */
     return -3;
 
-  if (strchr("\\'\"`", c))	/* \\ or \' or \" or \`  " */
+  if (strchr("\\'\"`", c))	/* \\ or \' or \" or \` */
     return c;
 
   if ((p = (char *) strchr(escape_symbol, c)))	/* \a \b \f \n \r \t \v */
@@ -582,7 +600,7 @@ Scan_Quoted_Char(StmInf *pstm, Bool convert, int c0)
  *-------------------------------------------------------------------------*/
 void
 Recover_After_Error(StmInf *pstm)
-#define Next_Char   Read_Next_Char(pstm,convert); if (c==EOF) return
+#define Next_Char   Read_Next_Char(pstm, convert); if (c == EOF) return
 {
   int c0;
   Bool convert;
@@ -663,21 +681,13 @@ Scan_Next_Atom(StmInf *pstm)
 {
   char *s;
 
-  for (;;)
-    {
-      Read_Next_Char(pstm, TRUE);
-      if (c_type != LA)		/* layout character */
-	break;
-    }
-
+  do
+    Read_Next_Char(pstm, TRUE);
+  while (c_type == LA);		/* layout character */
 
   token.line = pstm->line_count + 1;
   token.col = pstm->line_pos;
 
-  if (c_type == DQ
-      && Flag_Value(FLAG_DOUBLE_QUOTES) != FLAG_DOUBLE_QUOTES_ATOM)
-    c_type = 0;
-
   switch (c_type)
     {
     case SL:			/* small letter */
@@ -692,9 +702,16 @@ Scan_Next_Atom(StmInf *pstm)
       Unget_Last_Char;
       break;
 
-    case QT:			/* quote */
     case DQ:			/* double quote */
+      if ((Flag_Value(FLAG_DOUBLE_QUOTES) & FLAG_AS_PART_MASK) != FLAG_AS_ATOM)
+	goto error;
+      goto do_scan_quoted;
+
     case BQ:			/* back quote */
+      if ((Flag_Value(FLAG_BACK_QUOTES) & FLAG_AS_PART_MASK) != FLAG_AS_ATOM)
+	goto error;
+    case QT:			/* quote */
+    do_scan_quoted:
       err_msg = NULL;
       Scan_Quoted(pstm);
       if (err_msg)
@@ -718,6 +735,7 @@ Scan_Next_Atom(StmInf *pstm)
       break;
 
     default:
+    error:
       Unget_Last_Char;
       return "cannot start an atom (use quotes ?)";
     }
