@@ -6,7 +6,7 @@
  * Descr.: sort management                                                 * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: sort.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: sort.pl,v 1.7 2002/03/19 18:24:35 diaz Exp $ */
 
 :-	built_in.
 
@@ -51,7 +51,7 @@ keysort(List1, List2) :-
 	'$call_c_test'('Sort_List_2'(List1, List2)).
 
 
-        /* sort in-place predicates */
+        % sort in-place predicates
 
 sort(List) :-
 	set_bip_name(sort, 1),
