@@ -6,7 +6,7 @@
  * Descr.: stream selection and control management                         * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: stream.pl,v 1.9 2002/05/07 17:45:46 diaz Exp $ */
 
 :-	built_in.
 
@@ -88,7 +88,7 @@ set_output(SorA) :-
           %   0/1   0/1/2     0/1   0/1/2         0/1 0/1         0/1
           %   buffering       eof_action         reposition      text
           %   b8=specified    b5=specified       b2=specified    0=text
-          %   b7/6=if specif  b4/3=if specif     b1=if secif     1=binary
+          %   b7/6=if specif  b4/3=if specif     b1=if specified 1=binary
           %     buffering       eof action         reposition
           %     0=none          0=error            0=false
           %     1=line          1=eof_code         1=true
@@ -108,9 +108,14 @@ open(SourceSink, Mode, Stream, Options) :-
 '$open'(SourceSink, Mode, Stream, Options) :-
 	'$set_open_defaults',
 	'$get_open_stm'(Stream, Stm),
-	'$get_open_options'(Options, LAlias),
+	g_link('$open_aliases', LAlias),
+	g_link('$open_mirrors', LMirror),
+	'$get_open_options'(Options),
+	g_read('$open_aliases', []), % close the list
+	g_read('$open_mirrors', []), % close the list
 	'$call_c'('Open_3'(SourceSink, Mode, Stm)),
-	'$add_alias_to_stream'(LAlias, Stream).
+	'$add_aliases_to_stream'(LAlias, Stream),
+	'$add_mirrors_to_stream'(LMirror, Stream).
 
 
 
@@ -130,23 +135,23 @@ open(SourceSink, Mode, Stream, Options) :-
 
 
 
-'$get_open_options'(Options, LAlias) :-
+'$get_open_options'(Options) :-
 	'$check_list'(Options),
-	'$get_open_options1'(Options, [], LAlias).
+	'$get_open_options1'(Options).
 
 
-'$get_open_options1'([], LAlias, LAlias).
+'$get_open_options1'([]).
 
-'$get_open_options1'([X|Options], LAlias, LAlias2) :-
-	'$get_open_options2'(X, LAlias, LAlias1), !,
-	'$get_open_options1'(Options, LAlias1, LAlias2).
+'$get_open_options1'([X|Options]) :-
+	'$get_open_options2'(X), !,
+	'$get_open_options1'(Options).
 
 
-'$get_open_options2'(X, _, _) :-
+'$get_open_options2'(X) :-
 	var(X),
 	'$pl_err_instantiation'.
 
-'$get_open_options2'(type(X), LAlias, LAlias) :-
+'$get_open_options2'(type(X)) :-
 	nonvar(X),
 	(   X = text,
 	    '$sys_var_set_bit'(0, 0)
@@ -154,7 +159,7 @@ open(SourceSink, Mode, Stream, Options) :-
 	    '$sys_var_reset_bit'(0, 0)
 	).
 
-'$get_open_options2'(reposition(X), LAlias, LAlias) :-
+'$get_open_options2'(reposition(X)) :-
 	nonvar(X),
 	(   X = false,
 	    '$sys_var_reset_bit'(0, 1)
@@ -163,7 +168,7 @@ open(SourceSink, Mode, Stream, Options) :-
 	),
 	'$sys_var_set_bit'(0, 2).
 
-'$get_open_options2'(eof_action(X), LAlias, LAlias) :-
+'$get_open_options2'(eof_action(X)) :-
 	nonvar(X),
 	(   X = error,
 	    '$sys_var_reset_bit'(0, 4),
@@ -177,7 +182,7 @@ open(SourceSink, Mode, Stream, Options) :-
 	),
 	'$sys_var_set_bit'(0, 5).
 
-'$get_open_options2'(buffering(X), LAlias, LAlias) :-
+'$get_open_options2'(buffering(X)) :-
 	nonvar(X),
 	(   X = none,
 	    '$sys_var_reset_bit'(0, 7),
@@ -191,24 +196,41 @@ open(SourceSink, Mode, Stream, Options) :-
 	),
 	'$sys_var_set_bit'(0, 8).
 
-'$get_open_options2'(alias(X), LAlias, [X|LAlias]) :-
+'$get_open_options2'(alias(X)) :-
 	atom(X), !,
 	(   '$call_c_test'('Test_Alias_Not_Assigned_1'(X)) ->
-	    true
+	    g_read('$open_aliases', [X|End]),
+	    g_link('$open_aliases', End) % write new end variable
 	;   '$pl_err_permission'(open, source_sink, alias(X))
 	).
 
-'$get_open_options2'(X, _, _) :-
+'$get_open_options2'(mirror(X)) :-
+	nonvar(X),
+	(X = '$stream'(MStm), integer(MStm) ; atom(X)),
+	!,
+	'$call_c'('Check_Valid_Mirror_1'(X)),
+	g_read('$open_mirrors', [X|End]),
+	g_link('$open_mirrors', End). % write new end variable
+
+'$get_open_options2'(X) :-
 	'$pl_err_domain'(stream_option, X).
 
 
 
 
-'$add_alias_to_stream'([], _).
+'$add_aliases_to_stream'([], _).
 
-'$add_alias_to_stream'([Alias|LAlias], Stream) :-
+'$add_aliases_to_stream'([Alias|LAlias], Stream) :-
 	'$call_c'('Add_Stream_Alias_2'(Stream, Alias)),
-	'$add_alias_to_stream'(LAlias, Stream).
+	'$add_aliases_to_stream'(LAlias, Stream).
+
+
+
+'$add_mirrors_to_stream'([], _).
+
+'$add_mirrors_to_stream'([Mirror|LMirror], Stream) :-
+	'$call_c'('Add_Stream_Mirror_2'(Stream, Mirror)),
+	'$add_mirrors_to_stream'(LMirror, Stream).
 
 
 
@@ -279,6 +301,20 @@ add_stream_alias(_, Alias) :-
 
 
 
+add_stream_mirror(SorA, Mirror) :-
+	set_bip_name(add_stream_mirror, 2),
+	'$call_c'('Add_Stream_Mirror_2'(SorA, Mirror)).
+
+
+
+
+remove_stream_mirror(SorA, Mirror) :-
+	set_bip_name(remove_stream_mirror, 2),
+	'$call_c_test'('Remove_Stream_Mirror_2'(SorA, Mirror)).
+
+
+
+
 set_stream_type(SorA, Type) :-
 	set_bip_name(set_stream_type, 2),
 	(   var(Type) ->
@@ -397,6 +433,8 @@ stream_property(Stream, Property) :-
 
 '$check_stream_prop'(alias(_)).
 
+'$check_stream_prop'(mirror(_)).
+
 '$check_stream_prop'(type(_)).
 
 '$check_stream_prop'(reposition(_)).
@@ -430,6 +468,10 @@ stream_property(Stream, Property) :-
 '$stream_property1'(alias(Alias), S) :-
 	'$current_alias'(S, Alias).
 
+'$stream_property1'(mirror(MS), S) :-
+	'$current_mirror'(S, MStm),
+	MS = '$stream'(MStm).
+
 '$stream_property1'(type(Type), S) :-
 	'$call_c_test'('Stream_Prop_Type_2'(Type, S)).
 
@@ -484,6 +526,25 @@ current_alias(Stream, Alias) :-
 
 
 
+current_mirror(Stream, MStream) :-
+	set_bip_name(current_mirror, 2),
+	'$check_stream_or_var'(Stream, S),
+	'$check_stream_or_var'(MStream, MStm),
+	'$current_stream'(S),
+	'$current_mirror'(S, MStm).
+
+
+
+
+'$current_mirror'(S, MStm) :-
+	'$call_c_test'('Current_Mirror_2'(S, MStm)).
+
+'$current_mirror_alt' :-             % used by C code to create a choice-point
+	'$call_c_test'('Current_Mirror_Alt_0').
+
+
+
+
 stream_position(SorA, Position) :-
 	set_bip_name(stream_position, 2),
 	'$stream_position'(SorA, Position).
@@ -544,19 +605,16 @@ set_stream_line_column(SorA, Line, Col) :-
 
 
 
-          /*------------------------* 
-           * term_stream operations * 
-           *------------------------*/
+          % term_stream operations
 
 
           % term type (open_[input/output]_xxxx_stream/2) in sys_var[0]:
-          % 0=atom, 1=chars, 2=codes, 
-
+          % 1=atom, 2=chars, 3=codes
 
 open_input_atom_stream(SinkAtom, Stream) :-
 	set_bip_name(open_input_atom_stream, 2),
 	'$get_open_stm'(Stream, Stm),
-	'$sys_var_write'(0, 0),
+	'$sys_var_write'(0, 1),
 	'$call_c'('Open_Input_Term_Stream_2'(SinkAtom, Stm)).
 
 
@@ -565,7 +623,7 @@ open_input_atom_stream(SinkAtom, Stream) :-
 open_input_chars_stream(SinkChars, Stream) :-
 	set_bip_name(open_input_chars_stream, 2),
 	'$get_open_stm'(Stream, Stm),
-	'$sys_var_write'(0, 1),
+	'$sys_var_write'(0, 2),
 	'$call_c'('Open_Input_Term_Stream_2'(SinkChars, Stm)).
 
 
@@ -574,7 +632,7 @@ open_input_chars_stream(SinkChars, Stream) :-
 open_input_codes_stream(SinkCodes, Stream) :-
 	set_bip_name(open_input_codes_stream, 2),
 	'$get_open_stm'(Stream, Stm),
-	'$sys_var_write'(0, 2),
+	'$sys_var_write'(0, 3),
 	'$call_c'('Open_Input_Term_Stream_2'(SinkCodes, Stm)).
 
 
@@ -582,7 +640,7 @@ open_input_codes_stream(SinkCodes, Stream) :-
 
 close_input_atom_stream(SorA) :-
 	set_bip_name(close_input_atom_stream, 1),
-	'$sys_var_write'(0, 0),
+	'$sys_var_write'(0, 1),
 	'$call_c'('Close_Input_Term_Stream_1'(SorA)).
 
 
@@ -590,7 +648,7 @@ close_input_atom_stream(SorA) :-
 
 close_input_chars_stream(SorA) :-
 	set_bip_name(close_input_chars_stream, 1),
-	'$sys_var_write'(0, 1),
+	'$sys_var_write'(0, 2),
 	'$call_c'('Close_Input_Term_Stream_1'(SorA)).
 
 
@@ -598,7 +656,7 @@ close_input_chars_stream(SorA) :-
 
 close_input_codes_stream(SorA) :-
 	set_bip_name(close_input_codes_stream, 1),
-	'$sys_var_write'(0, 2),
+	'$sys_var_write'(0, 3),
 	'$call_c'('Close_Input_Term_Stream_1'(SorA)).
 
 
@@ -607,7 +665,7 @@ close_input_codes_stream(SorA) :-
 open_output_atom_stream(Stream) :-
 	set_bip_name(open_output_atom_stream, 1),
 	'$get_open_stm'(Stream, Stm),
-	'$sys_var_write'(0, 0),
+	'$sys_var_write'(0, 1),
 	'$call_c'('Open_Output_Term_Stream_1'(Stm)).
 
 
@@ -616,7 +674,7 @@ open_output_atom_stream(Stream) :-
 open_output_chars_stream(Stream) :-
 	set_bip_name(open_output_chars_stream, 1),
 	'$get_open_stm'(Stream, Stm),
-	'$sys_var_write'(0, 1),
+	'$sys_var_write'(0, 2),
 	'$call_c'('Open_Output_Term_Stream_1'(Stm)).
 
 
@@ -625,7 +683,7 @@ open_output_chars_stream(Stream) :-
 open_output_codes_stream(Stream) :-
 	set_bip_name(open_output_codes_stream, 1),
 	'$get_open_stm'(Stream, Stm),
-	'$sys_var_write'(0, 2),
+	'$sys_var_write'(0, 3),
 	'$call_c'('Open_Output_Term_Stream_1'(Stm)).
 
 
@@ -633,7 +691,7 @@ open_output_codes_stream(Stream) :-
 
 close_output_atom_stream(SorA, SinkAtom) :-
 	set_bip_name(close_output_atom_stream, 2),
-	'$sys_var_write'(0, 0),
+	'$sys_var_write'(0, 1),
 	'$call_c_test'('Close_Output_Term_Stream_2'(SorA, SinkAtom)).
 
 
@@ -641,7 +699,7 @@ close_output_atom_stream(SorA, SinkAtom) :-
 
 close_output_chars_stream(SorA, SinkChars) :-
 	set_bip_name(close_output_chars_stream, 2),
-	'$sys_var_write'(0, 1),
+	'$sys_var_write'(0, 2),
 	'$call_c_test'('Close_Output_Term_Stream_2'(SorA, SinkChars)).
 
 
@@ -649,5 +707,5 @@ close_output_chars_stream(SorA, SinkChars) :-
 
 close_output_codes_stream(SorA, SinkCodes) :-
 	set_bip_name(close_output_codes_stream, 2),
-	'$sys_var_write'(0, 2),
+	'$sys_var_write'(0, 3),
 	'$call_c_test'('Close_Output_Term_Stream_2'(SorA, SinkCodes)).
