@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: stream_supp.c,v 1.13 2002/06/12 11:53:14 diaz Exp $ */
+/* $Id: stream_supp.c,v 1.14 2002/09/19 11:00:35 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -903,11 +903,7 @@ Stdio_Is_Repositionable(FILE *f)
 {
   int fd = fileno(f);
 
-  if (isatty(fd) || lseek(fd, 0, SEEK_END) < 0)
-    return FALSE;
-
-  lseek(fd, 0, SEEK_SET);
-  return TRUE;
+  return !isatty(fd) && lseek(fd, 0, SEEK_CUR) >= 0;
 }
 
 
