@@ -6,7 +6,7 @@
  * Descr.: top Level                                                       * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: top_level.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: top_level.pl,v 1.9 2002/04/19 15:43:36 diaz Exp $ */
 
 :-	built_in.
 
@@ -52,6 +52,9 @@ break :-
 	;   true
 	),
 	'$sys_var_inc'(10),
+	g_read('$cmd_line_entry_goal', LGoal),
+	'$exec_cmd_line_entry_goals'(LGoal),
+	g_assign('$cmd_line_entry_goal', []),
 	'$top_level1',
 	'$sys_var_dec'(10),
 	'$sys_var_write'(11, B),
@@ -72,6 +75,7 @@ break :-
 
 
 '$top_level_abort' :-
+	'$reinit_after_exception',
 	'$sys_var_read'(11, B),
 	format(top_level_output, 'execution aborted~n', []),
 	'$catch_sync_for_fail_at'(B).
@@ -80,6 +84,7 @@ break :-
 
 
 '$top_level_stop' :-
+	'$reinit_after_exception',
 	'$sys_var_read'(11, B),
 	'$catch_sync_for_fail_at'(B).
 
@@ -87,13 +92,7 @@ break :-
 
 
 '$top_level_exception'(X) :-
-	(   '$sys_var_read'(12, 1) ->
-	    g_read('$user_prompt', UserPrompt),
-	    '$set_linedit_prompt'(UserPrompt)
-	;   true
-	),
-% g_read('$char_conv',CharConv),
-% set_prolog_flag(char_conversion,CharConv),
+	'$reinit_after_exception',
 	format(top_level_output, '~Nuncaught exception: ', []),
 	write_term(top_level_output, X, [quoted(true), numbervars(false)]),
 	nl(top_level_output),
@@ -101,15 +100,26 @@ break :-
 
 
 
+'$reinit_after_exception' :-
+% g_read('$char_conv', CharConv),
+% set_prolog_flag(char_conversion, CharConv),
+	(   '$sys_var_read'(12, 1) ->
+	    g_read('$user_prompt', UserPrompt),
+	    '$set_linedit_prompt'(UserPrompt)
+	;   true
+	).
+
+
+
 
 '$top_level2' :-
 	repeat,
 	'$get_current_B'(B),
 	'$sys_var_write'(11, B),
 	'$write_indicator',
-% current_prolog_flag(char_conversion,CharConv),
-% g_assign('$char_conv',CharConv),
-% set_prolog_flag(char_conversion,off),
+% current_prolog_flag(char_conversion, CharConv),
+% g_assign('$char_conv', CharConv),
+% set_prolog_flag(char_conversion, off),
 	Prompt = '| ?- ',
 	(   '$sys_var_read'(12, 1) ->
 	    '$get_linedit_prompt'(UserPrompt),
@@ -118,17 +128,17 @@ break :-
 	;   write(top_level_output, Prompt)
 	),
 	flush_output(top_level_output),
-	read_term(top_level_input, X, [variable_names(QueryVars)]),
+	'$read_query'(X, QueryVars),
 	(   '$sys_var_read'(12, 1) ->
 	    '$set_linedit_prompt'(UserPrompt)
 	;   true
 	),
-% set_prolog_flag(char_conversion,CharConv),
+% set_prolog_flag(char_conversion, CharConv),
 	sort(QueryVars, QueryVars1),
 	(   X == end_of_file ->
 	    nl(top_level_output), !
 	;   user_time(Time0),
-	    (   '$exec'(X, QueryVars1) ->
+	    (   '$exec_query'(X, QueryVars1) ->
 	        Ok = yes
 	    ;   Ok = no
 	    ),
@@ -175,7 +185,25 @@ break :-
 
 
 
-'$exec'(X, QueryVars) :-
+'$read_query'(X, QueryVars) :-
+	'$sys_var_read'(10, 1), % comment to execute in nested top-levels
+	g_read('$cmd_line_query_goal', [Goal|LGoal]),
+	g_assign('$cmd_line_query_goal', LGoal),
+	!,
+	Prompt = '| ?- ',
+	(   '$sys_var_read'(12, 1) -> 
+	    write(top_level_output, Prompt)
+	;   true),
+	format(top_level_output, '~a.~n', [Goal]),
+	read_term_from_atom(Goal, X, [end_of_term(eof), variable_names(QueryVars)]).
+
+'$read_query'(X, QueryVars) :-
+	read_term(top_level_input, X, [variable_names(QueryVars)]).
+
+
+
+
+'$exec_query'(X, QueryVars) :-
 	g_read('$debug_mode', DebugMode),
 	g_assign('$debug_next', DebugMode),
 	g_assign('$all_solutions', f),
@@ -183,13 +211,12 @@ break :-
 	'$call'(X, top_level, 0, true),
 	'$get_current_B'(B1),
 	format(top_level_output, '~N', []),
+	'$set_query_vars_names'(QueryVars, ToDispVars1),
 	(   fail,                             % do not activate 'alt if vars'
-	    QueryVars = [] ->
+	    ToDispVars1 = [] ->
 	    true                              % no alt if only anonymous vars
-	;   name_query_vars(QueryVars, ToDispVars),
-	    name_singleton_vars(ToDispVars),
-	    bind_variables(ToDispVars, [exclude(QueryVars), namevars]),
-	    '$write_solution'(ToDispVars, B1, B),
+	;
+	    '$write_solution'(ToDispVars1, B1, B),
 	    (   B1 > B ->
 	        g_read('$all_solutions', f),          % fail for previous 'a'
 	        write(top_level_output, ' ? '),
@@ -201,6 +228,30 @@ break :-
 
 
 
+'$set_query_vars_names'(QueryVars, ToDispVars) :-
+	name_query_vars(QueryVars, ToDispVars),
+	'$remove_underscore_vars'(ToDispVars, ToDispVars1),
+	name_singleton_vars(ToDispVars1),
+	bind_variables(ToDispVars1, [exclude(QueryVars), namevars]).
+
+
+
+
+'$remove_underscore_vars'([], []).
+
+'$remove_underscore_vars'([Name = Term|ToDispVars], ToDispVars1) :-
+	(   sub_atom(Name, 0, 1, _, '_')
+	;   Term = '$VARNAME'(Name1),
+	    sub_atom(Name1, 0, 1, _, '_')),
+	!,
+	'$remove_underscore_vars'(ToDispVars, ToDispVars1).
+	
+'$remove_underscore_vars'([X|ToDispVars], [X|ToDispVars1]) :-
+	'$remove_underscore_vars'(ToDispVars, ToDispVars1).
+
+
+
+
 '$write_solution'([], B1, B) :-
 	!,
 	(   B1 > B ->
@@ -224,7 +275,7 @@ break :-
 
 '$read_return' :-
 	flush_output(top_level_output),
-	get_code_no_echo(top_level_input, X),
+	get_key(top_level_input, X),
 	'$read_return'(X), !.
 
 
@@ -246,3 +297,32 @@ break :-
 	nl(top_level_output),
 	write(top_level_output, 'Action (; for next solution, a for all solutions, RET to stop) ? '),
 	'$read_return'.
+
+
+
+/* interface with command-line options executing goals */
+
+'$exec_cmd_line_entry_goals'([Goal|LGoal]):-
+	!,
+	'$exec_cmd_line_goal'(Goal),
+	'$exec_cmd_line_entry_goals'(LGoal).
+
+'$exec_cmd_line_entry_goals'(_).		% can be another term than []
+
+
+						
+
+'$exec_cmd_line_goal'(Goal) :-		% called by top_level.c
+	(   '$catch'('$exec_cmd1'(Goal), Err, '$exec_cmd_err'(Goal, Err), 'command-line', -1, false) ->
+	    true
+	;   format('~Nwarning: command-line goal ~q failed~n', [Goal])).
+
+
+'$exec_cmd1'(Goal) :-
+	read_term_from_atom(Goal, TermGoal, [end_of_term(eof)]),
+	'$call'(TermGoal, 'command-line', -1, false).
+
+
+'$exec_cmd_err'(Goal, Err) :-
+	format('~Nwarning: command-line goal ~q caused exception: ~q~n', [Goal, Err]).
+
