@@ -6,7 +6,7 @@
  * Descr.: unification management                                          * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: unify.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: unify.pl,v 1.7 2002/03/19 18:24:36 diaz Exp $ */
 
 :-	built_in.
 
@@ -35,7 +35,7 @@ X = X.
 
 
 unify_with_occurs_check(X, Y) :-
-	'$call_c_test'('Unify_Occurs_Check'(X, Y)).
+	'$call_c'('Unify_Occurs_Check'(X, Y), [boolean, fast_call]).
 
 
 
