@@ -6,7 +6,7 @@
  * Descr.: term output (write/1 and friends) management                    * 
  * Author: Daniel Diaz                                                     * 
  *                                                                         * 
- * Copyright (C) 1999-2001 Daniel Diaz                                     * 
+ * Copyright (C) 1999-2002 Daniel Diaz                                     * 
  *                                                                         * 
  * GNU Prolog is free software; you can redistribute it and/or modify it   * 
  * under the terms of the GNU General Public License as published by the   * 
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     * 
  *-------------------------------------------------------------------------*/
 
-/* $Id: write.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: write.pl,v 1.8 2002/04/12 14:36:26 diaz Exp $ */
 
 :-	built_in.
 
@@ -78,8 +78,7 @@ display(SorA, Term) :-
 
 
 
-          /* see file print.pl for the definition of print/1-2 */
-
+          % see file print.pl for the definition of print/1-2
 
 
 
@@ -93,6 +92,7 @@ display(SorA, Term) :-
           %  1=true     1=true     1=true     1=true       1=true    1=true
           %
           % max_depth in sys_var[1]
+          % priority in sys_var[2]
 
 
 write_term(Term, Options) :-
@@ -113,7 +113,8 @@ write_term(SorA, Term, Options) :-
 
 '$set_write_defaults' :-
 	'$sys_var_write'(0, 12),                               % default mask
-	'$sys_var_write'(1, -1).
+	'$sys_var_write'(1, -1),
+	'$sys_var_write'(2, 1200).
 
 
 
@@ -186,6 +187,10 @@ write_term(SorA, Term, Options) :-
 	integer(X),
 	'$sys_var_write'(1, X).
 
+'$get_write_options2'(priority(X)) :-
+	integer(X),
+	'$sys_var_write'(2, X).
+
 '$get_write_options2'(X) :-
 	'$pl_err_domain'(write_option, X).
 
