@@ -6,7 +6,7 @@
  * Descr.: general engine                                                  *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: engine1.c,v 1.4 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: engine1.c,v 1.6 2002/05/07 17:45:47 diaz Exp $ */
 
 #include "engine_pl.h"
 
@@ -83,6 +83,11 @@ Call_Compiled(CodePtr codep)
   register WamWord *rb asm("15") = reg_bank;
   ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
 
+#elif defined(M_powerpc_darwin)
+
+  register WamWord *rb asm("r15") = reg_bank;
+  ensure_reserved = (WamWord *) rb; /* to avoid gcc warning */
+
 #elif defined(M_sparc)
 
   register WamWord *rb asm("%l0") = reg_bank;
