@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine.c,v 1.13 2002/05/07 17:45:47 diaz Exp $ */
+/* $Id: machine.c,v 1.15 2002/09/17 13:04:01 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -786,8 +786,18 @@ M_Real_Time(void)
 void
 M_Randomize(void)
 {
-  M_Set_Seed(M_Real_Time());
-  M_Set_Seed(M_Random_Integer(256));
+#ifdef M_ix86_win32
+  int seed = GetTickCount();
+#else
+  struct timeval tv;
+  int seed;
+
+  gettimeofday(&tv, NULL);
+  seed = ((tv.tv_sec * 1000) + (tv.tv_usec / 1000));
+#endif
+  seed = (seed ^ getpid()) & 0xFFFFFF;
+
+  M_Set_Seed(seed);
 }
 
 
