@@ -6,7 +6,7 @@
  * Descr.: machine dependent features                                      *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine1.c,v 1.6 2001/10/07 13:49:48 spa Exp $ */
+/* $Id: machine1.c,v 1.12 2002/06/12 11:58:36 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,6 +50,7 @@
 #include <sys/stat.h>
 #include <sys/utsname.h>
 #include <sys/param.h>
+#include <time.h>
 #else
 #include <windows.h>
 #include <process.h>
@@ -237,18 +238,19 @@ M_Cmd_Line_To_Argv(char *cmd, int *argc)
 	}
       arg[i++] = p;
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-      if (*p == '"')
+      while (*p != ' ' && *p != '\t' && *p != '\0')
 	{
-	  while (*p != '"' && *p != '\0')
-	    p++;
 	  if (*p == '"')
+	  {
+	    do
+	      p++;
+	    while (*p != '"' && *p != '\0');
+	    if (*p == '"')
+	      p++;
+	  }
+	  else
 	    p++;
 	}
-      else
-#endif
-	while (*p != ' ' && *p != '\t' && *p != '\0')
-	  p++;
 
       if (*p == '\0')
 	break;
@@ -577,22 +579,73 @@ M_Get_Status(int pid)
  *                                                                         *
  *-------------------------------------------------------------------------*/
 char *
-M_Mktemp(char *template)
+M_Mktemp(char *tmpl)
 {
-#ifdef HAVE_MKSTEMP
+				/* redefined to avoid link warning */
+#if defined(__unix__) || defined(__CYGWIN__)
+				/* this code comes from glibc */
+  int len;
+  char *XXXXXX;
+  static unsigned long value;
+  int count;
+  struct stat buf;
+  static const char letters[] =
+    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 
-  int fd = mkstemp(template);
-  if (fd == -1)
-    return NULL;
+#ifndef TMP_MAX
+#define TMP_MAX 238328
+#endif
 
-  close(fd);
-  unlink(template);		/* don't leave it sitting around... */
-  return template;
+  len = strlen (tmpl);
+  if (len < 6 || strcmp(&tmpl[len - 6], "XXXXXX"))
+    {
+      errno = EINVAL;
+      return NULL;
+    }
+
+  /* This is where the Xs start.  */
+  XXXXXX = &tmpl[len - 6];
+
+  value += (unsigned long) time(NULL) ^ getpid();
+
+  for (count = 0; count < TMP_MAX; value += 7777, ++count)
+    {
+      unsigned long v = value;
+
+      /* Fill in the random bits.  */
+      XXXXXX[0] = letters[v % 62];
+      v /= 62;
+      XXXXXX[1] = letters[v % 62];
+      v /= 62;
+      XXXXXX[2] = letters[v % 62];
+      v /= 62;
+      XXXXXX[3] = letters[v % 62];
+      v /= 62;
+      XXXXXX[4] = letters[v % 62];
+      v /= 62;
+      XXXXXX[5] = letters[v % 62];
+
+      if (lstat(tmpl, &buf) < 0)
+	{
+	  if (errno == ENOENT)
+	    {
+	      errno = 0;
+	      return tmpl;
+	    }
+	  else
+	    /* Give up now. */
+	    return NULL;
+	}
+    }
+
+  /* We got out of the loop because we ran out of combinations to try.  */
+  errno = EEXIST;
+  return NULL;
 
 #else
 
   errno = 0;
-  return mktemp(template);
+  return mktemp(tmpl);
 
 #endif
 }
@@ -607,17 +660,20 @@ M_Mktemp(char *template)
 char *
 M_Tempnam(char *dir, char *pfx)
 {
-#ifdef HAVE_MKSTEMP
+#if defined(__unix__) || defined(__CYGWIN__)
+				/* this code comes from glibc */
   char tmpl[MAXPATHLEN];
   char *d;
   int dlen, plen;
-  int fd;
   struct stat buf;
 
+#ifndef P_tmpdir
+#define P_tmpdir "/tmp"
+#endif
+
 #define Dir_Exists(dir) (stat(dir, &buf) == 0 && S_ISDIR (buf.st_mode))
 
 
-				/* this code comes from glibc */
   if (!pfx || !pfx[0])
     {
       pfx = "file";
@@ -663,14 +719,10 @@ M_Tempnam(char *dir, char *pfx)
     }
 
   sprintf(tmpl, "%.*s/%.*sXXXXXX", dlen, dir, plen, pfx);
-
-  fd = mkstemp(tmpl);
-  if (fd == -1)
-     return NULL;
-
-  close(fd);
-  unlink(tmpl);
-  return strdup(tmpl);
+  d = M_Mktemp(tmpl);
+  if (d)
+    d = strdup(d);
+  return d;
 
 #else
 
@@ -697,53 +749,89 @@ M_Tempnam(char *dir, char *pfx)
 #if defined(__unix__) || defined(__CYGWIN__)
 #define PREFIX_DIR
 #else
-#define PREFIX_DIR "\\bin\\"
+#define PREFIX_DIR "c:\\cygwin\\bin\\"
 #endif
 
-#define READ(str,f) \
-{ \
- char buff[1024]; \
- DBGPRINTF("\n   Reading redirected %s\n",str); \
- while(fgets(buff,sizeof(buff),f)) \
-    { \
-     if (buff[strlen(buff)-1]=='\n') \
-         buff[strlen(buff)-1]='\0'; \
-     DBGPRINTF("   <%s>\n",buff); \
-if (feof(f)) break; \
-    } \
- fclose(f); \
- DBGPRINTF("   End reading redirected %s\n",str); \
+
+#define READ(str, f)					\
+{							\
+  char buff[1024];					\
+ 							\
+  DBGPRINTF("\n   Reading redirected %s\n", str);	\
+  while (fgets(buff, sizeof(buff), f))			\
+    {							\
+      if (buff[strlen(buff) - 1] == '\n')		\
+        buff[strlen(buff) - 1] = '\0';			\
+      DBGPRINTF("   <%s>\n", buff);			\
+      if (feof(f))					\
+        break;						\
+    }							\
+  fclose(f);						\
+  DBGPRINTF("   End reading redirected %s\n", str);	\
 }
 
-#define CHECK(pid) \
-{ \
- if (pid== -1) \
-    { \
-     DBGPRINTF("   ERROR executing Spawn: errno=%d\n",errno); \
-     exit(1); \
-    } \
- if (pid== -2) \
-    { \
-     DBGPRINTF(   "ERROR executing Spawn: unknown error\n"); \
-     exit(1); \
-    } \
- DBGPRINTF("   pid=%d (%x)\n",pid,pid); \
+
+#define CHECK(pid)						\
+{								\
+  if (pid == -1)						\
+    {								\
+      DBGPRINTF("   ERROR executing Spawn: errno=%d\n", errno);	\
+      exit(1);							\
+    }								\
+  if (pid == -2)						\
+    {								\
+      DBGPRINTF("ERROR executing Spawn: unknown error\n");	\
+      exit(1);							\
+    }								\
+  DBGPRINTF("   pid=%d (%x)\n", pid, pid);			\
 }
 
-#define STAT(pid) \
-{ \
- int status=M_Get_Status(pid); \
- STATUS(status) \
+
+#define STAT(pid)				\
+{						\
+  int status = M_Get_Status(pid);		\
+  STATUS(status)				\
 }
 
-#define STATUS(status) \
-{ \
- DBGPRINTF("   status=%d",status); \
- if (status== -1) \
-     DBGPRINTF(" errno=%d",errno); \
- DBGPRINTF("\n\n"); \
+#define STATUS(status)				\
+{						\
+  DBGPRINTF("   status=%d", status);		\
+  if (status == -1)				\
+    DBGPRINTF(" errno=%d", errno);		\
+  DBGPRINTF("\n\n");				\
 }
 
+
+#if 1
+#define COMMAND								\
+  strcpy(buff, PREFIX_DIR "bc --q");    /* should be modifiable */	\
+  arg[0] = buff;							\
+  arg[1] = (char *) 1;
+#else
+#define COMMAND
+ arg[0]=PREFIX_DIR "bc";			\
+ arg[1]="-q";					\
+ arg[2]=NULL;
+#endif
+
+#define CDE_STRING "1+255\n$foo\n2^10\nquit\n"
+#define CDE_INPUT  fprintf(i, CDE_STRING); fclose(i);
+
+
+#if 0
+#define POLL
+#include <sys/poll.h>
+#endif
+#ifdef POLL
+  {
+    int fd = fileno(i);
+    struct pollfd ufd = { fd, 7, 0 };
+    int r = poll(&ufd, 1, 100);
+    DBGPRINTF("poll ret:%d on fd %d returned events :%x\n", r, fd, ufd.revents);
+    return 0;
+  }
+#endif
+
 int
 main(int argc, char *argv[])
 {
@@ -793,65 +881,56 @@ main(int argc, char *argv[])
   strcpy(buff, PREFIX_DIR "uname -a");	/* should be modifiable */
   arg[0] = buff;
   arg[1] = (char *) 1;
-  pid = M_Spawn_Redirect(arg, 0, &i, &o, NULL);
+  pid = M_Spawn_Redirect(arg, 0, NULL, &o, NULL);
   CHECK(pid);
   READ("output", o);
   STAT(pid);
 #endif
 
-  DBGPRINTF("Executing gdb: p /x 12, xxx (unknown command) and p 0xFF\n");
+  COMMAND;
+  DBGPRINTF("Command is: %s with following input:\n" CDE_STRING,
+	    arg[0]);
+  DBGPRINTF("--- end of input\n");
+
+
 
 #if 1
-#define GDB_COMMAND \
- strcpy(buff,PREFIX_DIR "gdb --nw --quiet");    /* should be modifiable */ \
- arg[0]=buff; \
- arg[1]=(char *) 1;
-#else
-#define GDB_COMMAND \
- arg[0]=PREFIX_DIR "gdb"; \
- arg[1]="--nw"; \
- arg[2]="--quiet"; \
- arg[3]=NULL;
-#endif
-
-#define GDB_INPUT  fprintf(i,"p /x 12\nxxx\np 0xFF\nquit\n"); fclose(i);
-
-#if 1
-  DBGPRINTF("3- gdb with redirected input\n");
-  GDB_COMMAND;
+  DBGPRINTF("3- command with redirected input\n");
+  COMMAND;
   pid = M_Spawn_Redirect(arg, 0, &i, NULL, NULL);
   CHECK(pid);
-  GDB_INPUT;
+    
+  CDE_INPUT;
   STAT(pid);
 #endif
 
 #if 1
-  DBGPRINTF("4- gdb with redirected input and output\n");
-  GDB_COMMAND;
+  DBGPRINTF("4- command with redirected input and output\n");
+  COMMAND;
   pid = M_Spawn_Redirect(arg, 0, &i, &o, NULL);
   CHECK(pid);
-  GDB_INPUT;
+  CDE_INPUT;
   READ("output", o);
   STAT(pid);
 #endif
 
 #if 1
-  DBGPRINTF("5- gdb with redirected input output and error\n");
-  GDB_COMMAND;
+  DBGPRINTF("5- command with redirected input output and error\n");
+  COMMAND;
   pid = M_Spawn_Redirect(arg, 0, &i, &o, &e);
   CHECK(pid);
-  GDB_INPUT;
+  CDE_INPUT;
   READ("output", o);
   READ("error", e);
   STAT(pid);
 #endif
 
 #if 1
-  DBGPRINTF("6- gdb with redirected input and output=error\n");
-  GDB_COMMAND;
+  DBGPRINTF("6- command with redirected input and output=error\n");
+  COMMAND;
   pid = M_Spawn_Redirect(arg, 0, &i, &o, &o);
   CHECK(pid);
-  GDB_INPUT;
+  CDE_INPUT;
   READ("output/error", o);
   STAT(pid);
 #endif
