@@ -6,7 +6,7 @@
  * Descr.: machine dependent features - Header file                        *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: machine1.h,v 1.4 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: machine1.h,v 1.6 2002/03/19 18:24:36 diaz Exp $ */
 
 #include <stdio.h>
 
@@ -83,7 +83,7 @@ int M_Get_Status(int pid);
 
 
 
-char *M_Mktemp(char *template);
+char *M_Mktemp(char *tmp_template);
 
 char *M_Tempnam(char *dir, char *pfx);
 
