@@ -6,7 +6,7 @@
  * Descr.: main                                                            *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,11 +22,10 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: main.c,v 1.5 2001/04/27 11:26:55 spa Exp $ */
+/* $Id: main.c,v 1.9 2002/06/12 11:58:36 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
-#include <locale.h>
 
 #include "engine_pl.h"
 
@@ -53,9 +52,25 @@
 /*-------------------------------------------------------------------------*
  * MAIN                                                                    *
  *                                                                         *
+ * A problem appeared in GCC 3.0.x under Linux/ix86:                       *
+ * the main() function always use a frame (and thus ebp). This causes a bug*
+ * if ebp is used by gcc between Start_Prolog() and Stop_Prolog() (e.g. to *
+ * access argv/argc or local variables) since ebp contains a WAM register. *
+ * Note that after Stop_Prolog() all registers are restored and ebp is     *
+ * correct when returning in main().                                       *
+ *                                                                         *
+ * To solve we can use an intermediate function Main_Wrapper() called by   *
+ * the main() function.                                                    *
+ *                                                                         *
+ * Another solution consists in passing -mpreferred-stack-boundary=2 to gcc*
+ * since it gcc uses ebp to ensure the stack alignment (to 4).             *
+ *                                                                         *
+ * This main function uses the wrapper even if ebp is not really used      *
+ * between Start_Prolog() and Stop_Prolog() but to serve as model.         *
  *-------------------------------------------------------------------------*/
-int
-main(int argc, char *argv[])
+
+static int
+Main_Wrapper(int argc, char *argv[])
 {
   int nb_user_directive;
   Bool top_level;
@@ -66,17 +81,22 @@ main(int argc, char *argv[])
 
   Stop_Prolog();
 
-  if (top_level)
-    return 0;
-
-  if (nb_user_directive)
+  if (top_level || nb_user_directive)
     return 0;
 
   fprintf(stderr,
-	  "Warning: no initial goal executed\n"
-	  "   use a directive :- initialization(Goal)\n"
-	  "   or remove the link option --no-top-level"
-	  " (or --min-bips or --min-size)\n");
+          "Warning: no initial goal executed\n"
+          "   use a directive :- initialization(Goal)\n"
+          "   or remove the link option --no-top-level"
+          " (or --min-bips or --min-size)\n");
 
   return 1;
 }
+
+
+int
+main(int argc, char *argv[])
+{
+  Exit_With_Value(Main_Wrapper(argc, argv));
+  return 0;			/* anything for the compiler */
+}
