@@ -6,7 +6,7 @@
  * Descr.: translation file for Linux/OSF on alpha                         *
  * Author: Alexander Diemand, Daniel Diaz                                  *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: alpha_any.c,v 1.5 2001/01/23 11:38:20 diaz Exp $ */
+/* $Id: alpha_any.c,v 1.8 2002/05/07 17:45:47 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -677,11 +677,11 @@ Pl_Jump(char *label)
 
 
 /*-------------------------------------------------------------------------*
- * PL_CALL                                                                 *
+ * PREP_CP                                                                 *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Call(char *label)
+Prep_CP(void)
 {
 #ifdef MAP_REG_CP
   Inst_Printf("lda", "%s,$Lcont%d", asm_reg_cp, w_label);	/* CP = $Lcont%d */
@@ -689,15 +689,38 @@ Pl_Call(char *label)
   Inst_Printf("lda", "$4,$Lcont%d", w_label);	/* CP = $Lcont%d */
   Inst_Printf("stq", "$4,%s", asm_reg_cp);
 #endif
+}
 
-  Pl_Jump(label);
 
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
   Label_Printf("$Lcont%d:", w_label++);
 
   /* get GP back */
   /* this has only be introduced to get dynamic object loading work */
   /* Inst_Printf("ldgp","$gp,0($27)"); */
+}
 
+
+
+
+/*-------------------------------------------------------------------------*
+ * PL_CALL                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Call(char *label)
+{
+  Prep_CP();
+  Pl_Jump(label);
+  Here_CP();
 }
 
 
@@ -776,10 +799,10 @@ void
 Move_From_Reg_Y(int index)
 {
 #ifdef MAP_REG_E
-  Inst_Printf("ldq", "$1,-%d(%s)", (index + 4) * 8, asm_reg_e);
+  Inst_Printf("ldq", "$1,%d(%s)", Y_OFFSET(index), asm_reg_e);
 #else
   Inst_Printf("ldq", "$4,%s", asm_reg_e);
-  Inst_Printf("ldq", "$1,-%d($4)", (index + 4) * 8);
+  Inst_Printf("ldq", "$1,%d($4)", Y_OFFSET(index));
 #endif
 
 }
@@ -808,10 +831,10 @@ void
 Move_To_Reg_Y(int index)
 {
 #ifdef MAP_REG_E
-  Inst_Printf("stq", "$1,-%d(%s)", (index + 4) * 8, asm_reg_e);
+  Inst_Printf("stq", "$1,%d(%s)", Y_OFFSET(index), asm_reg_e);
 #else
   Inst_Printf("ldq", "$4,%s", asm_reg_e);
-  Inst_Printf("stq", "$1,-%d($4)", (index + 4) * 8);
+  Inst_Printf("stq", "$1,%d($4)", Y_OFFSET(index));
 #endif
 }
 
@@ -1096,19 +1119,19 @@ Call_C_Arg_Reg_Y(int offset, int adr_of, int index)
   if (!adr_of)
     {
 #ifdef MAP_REG_E
-      Inst_Printf("ldq", "%s,-%d(%s)", dest, (index + 4) * 8, asm_reg_e);
+      Inst_Printf("ldq", "%s,%d(%s)", dest, Y_OFFSET(index), asm_reg_e);
 #else
       Inst_Printf("ldq", "$4,%s", asm_reg_e);
-      Inst_Printf("ldq", "%s,-%d($4)", dest, (index + 4) * 8);
+      Inst_Printf("ldq", "%s,%d($4)", dest, Y_OFFSET(index));
 #endif
     }
   else
     {
 #ifdef MAP_REG_E
-      Inst_Printf("lda", "%s,-%d(%s)", dest, (index + 4) * 8, asm_reg_e);
+      Inst_Printf("lda", "%s,%d(%s)", dest, Y_OFFSET(index), asm_reg_e);
 #else
       Inst_Printf("ldq", "$4,%s", asm_reg_e);
-      Inst_Printf("lda", "%s,-%d($4)", dest, (index + 4) * 8);
+      Inst_Printf("lda", "%s,%d($4)", dest, Y_OFFSET(index));
 #endif
     }
   if (offset > 5)
@@ -1355,12 +1378,11 @@ void
 Move_Ret_To_Reg_Y(int index)
 {				/* same as Move_To_Reg_Y */
 #ifdef MAP_REG_E
-  Inst_Printf("stq", "$0,-%d(%s)", (index + 4) * 8, asm_reg_e);
+  Inst_Printf("stq", "$0,%d(%s)", Y_OFFSET(index), asm_reg_e);
 #else
   Inst_Printf("ldq", "$4,%s", asm_reg_e);
-  Inst_Printf("stq", "$0,-%d($4)", (index + 4) * 8);
+  Inst_Printf("stq", "$0,%d($4)", Y_OFFSET(index));
 #endif
-
 }
 
 
