@@ -6,7 +6,7 @@
  * Descr.: translation file                                                *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ma2asm_inst.c,v 1.5 2001/01/23 11:38:21 diaz Exp $ */
+/* $Id: ma2asm_inst.c,v 1.8 2002/05/07 17:45:47 diaz Exp $ */
 
 #define MAPPER_FILE
 
@@ -31,14 +31,19 @@
 
 #include "../EnginePl/gp_config.h"
 #include "../EnginePl/wam_regs.h"
+#define FRAMES_ONLY
+#include "../EnginePl/wam_inst.h"
 #include "../EnginePl/machine.h"
 #include "../EnginePl/pl_params.h"
 #include "../EnginePl/obj_chain.h"
 
+#define Y_OFFSET(index)   ((- ENVIR_STATIC_SIZE - 1 - index) * sizeof(long))
+
 
 #if 0				/* to force the inclusion of a mapper */
 
-#include "ix86_any.c"
+#define M_powerpc_darwin
+#include "powerpc_any.c"
 
 #else
 
@@ -64,11 +69,11 @@
 
 #include "mips_irix.c"
 
-#elif defined(M_powerpc_linux)
+#elif defined(M_powerpc)
 
-#include "ppc_linux.c"
+#include "powerpc_any.c"
 
 #endif
 
-
 #endif
+
