@@ -6,7 +6,7 @@
  * Descr.: code generation - header file                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,14 +22,14 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: ma_protos.h,v 1.6 2001/01/23 11:38:21 diaz Exp $ */
+/* $Id: ma_protos.h,v 1.9 2002/05/07 17:45:47 diaz Exp $ */
 
 #if 0
 #define CHECK_PRINTF_ARGS
 #endif
 
 #ifdef CHECK_PRINTF_ARGS
-#define GCCPRINTF(x) __attribute__((format(printf,x,x+1)))
+#define GCCPRINTF(x) __attribute__((format(printf, x, x + 1)))
 #else
 #define GCCPRINTF(x)
 #endif
@@ -119,6 +119,10 @@ void Reload_E_In_Register(void);
 
 void Pl_Jump(char *label);
 
+void Prep_CP(void);
+
+void Here_CP(void);
+
 void Pl_Call(char *label);
 
 void Pl_Fail(void);
