@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: mips_irix.c,v 1.4 2000/12/19 17:43:08 diaz Exp $ */
+/* $Id: mips_irix.c,v 1.6 2002/05/07 17:45:47 diaz Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -316,12 +316,14 @@ Pl_Jump(char *label)
 }
 
 
+
+
 /*-------------------------------------------------------------------------*
- * PL_CALL                                                                 *
+ * PREP_CP                                                                 *
  *                                                                         *
  *-------------------------------------------------------------------------*/
 void
-Pl_Call(char *label)
+Prep_CP(void)
 {
 #ifdef MAP_REG_CP
   Inst_Printf("la", "%s,.Lcont%d", asm_reg_cp, w_label);	/* CP = .Lcont%d */
@@ -329,14 +331,39 @@ Pl_Call(char *label)
   Inst_Printf("la", "$13,.Lcont%d", w_label);
   Inst_Printf("sw", "$13,%s", asm_reg_cp);	/* CP = .Lcont%d */
 #endif
-  Pl_Jump(label);
+}
 
+
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
   Label_Printf(".Lcont%d:", w_label++);
 }
 
 
 
 
+/*-------------------------------------------------------------------------*
+ * PL_CALL                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Pl_Call(char *label)
+{
+  Prep_CP();
+  Pl_Jump(label);
+  Here_CP();
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_FAIL                                                                 *
  *                                                                         *
@@ -411,10 +438,10 @@ void
 Move_From_Reg_Y(int index)
 {
 #ifdef MAP_REG_E
-  Inst_Printf("lw", "$24,-%d(%s)", index * 4 + 16, asm_reg_e);
+  Inst_Printf("lw", "$24,%d(%s)", Y_OFFSET(index), asm_reg_e);
 #else
   Inst_Printf("lw", "$13,%s", asm_reg_e);
-  Inst_Printf("lw", "$24,-%d($13)", index * 4 + 16);
+  Inst_Printf("lw", "$24,%d($13)", Y_OFFSET(index));
 #endif
 }
 
@@ -442,10 +469,10 @@ void
 Move_To_Reg_Y(int index)
 {
 #ifdef MAP_REG_E
-  Inst_Printf("sw", "$24,-%d(%s)", index * 4 + 16, asm_reg_e);
+  Inst_Printf("sw", "$24,%d(%s)", Y_OFFSET(index), asm_reg_e);
 #else
   Inst_Printf("lw", "$13,%s", asm_reg_e);
-  Inst_Printf("sw", "$24,-%d($13)", index * 4 + 16);
+  Inst_Printf("sw", "$24,%d($13)", Y_OFFSET(index));
 #endif
 }
 
@@ -766,19 +793,19 @@ Call_C_Arg_Reg_Y(int offset, int adr_of, int index)
   if (!adr_of)
     {
 #ifdef MAP_REG_E
-      Inst_Printf("lw", "%s,-%d(%s)", dest, index * 4 + 16, asm_reg_e);
+      Inst_Printf("lw", "%s,%d(%s)", dest, Y_OFFSET(index), asm_reg_e);
 #else
       Inst_Printf("lw", "$12,%s", asm_reg_e);
-      Inst_Printf("lw", "%s,-%d($12)", dest, index * 4 + 16);
+      Inst_Printf("lw", "%s,%d($12)", dest, Y_OFFSET(index));
 #endif
     }
   else
     {
 #ifdef MAP_REG_E
-      Inst_Printf("la", "%s,-%d(%s)", dest, index * 4 + 16, asm_reg_e);
+      Inst_Printf("la", "%s,%d(%s)", dest, Y_OFFSET(index), asm_reg_e);
 #else
       Inst_Printf("lw", "$12,%s", asm_reg_e);
-      Inst_Printf("la", "%s,-%d($12)", dest, index * 4 + 16);
+      Inst_Printf("la", "%s,%d($12)", dest, Y_OFFSET(index));
 #endif
     }
   if (offset > 7)
@@ -1041,10 +1068,10 @@ void
 Move_Ret_To_Reg_Y(int index)
 {				/* same as Move_To_Reg_Y */
 #ifdef MAP_REG_E
-  Inst_Printf("sw", "$2,-%d(%s)", index * 4 + 16, asm_reg_e);
+  Inst_Printf("sw", "$2,%d(%s)", Y_OFFSET(index), asm_reg_e);
 #else
   Inst_Printf("lw", "$13,%s", asm_reg_e);
-  Inst_Printf("sw", "$2,-%d($13)", index * 4 + 16);
+  Inst_Printf("sw", "$2,%d($13)", Y_OFFSET(index));
 #endif
 }
 
