@@ -6,7 +6,7 @@
  * Descr.: translation file for SunOs/Solaris on sparc                     *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: sparc_any.c,v 1.6 2001/01/23 11:38:21 diaz Exp $ */
+/* $Id: sparc_any.c,v 1.9 2002/05/07 17:45:47 diaz Exp $ */
 
 #include <stdio.h>
 #include <string.h>
@@ -242,6 +242,34 @@ Pl_Jump(char *label)
 
 
 
+/*-------------------------------------------------------------------------*
+ * PREP_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Prep_CP(void)
+{
+  fprintf(stderr, "PREP_CP not implemented\n");
+  exit(1);
+}
+
+
+
+
+/*-------------------------------------------------------------------------*
+ * HERE_CP                                                                 *
+ *                                                                         *
+ *-------------------------------------------------------------------------*/
+void
+Here_CP(void)
+{
+  fprintf(stderr, "HERE_CP not implemented\n");
+  exit(1);
+}
+
+
+
+
 /*-------------------------------------------------------------------------*
  * PL_CALL                                                                 *
  *                                                                         *
@@ -334,7 +362,7 @@ Move_From_Reg_X(int index)
 void
 Move_From_Reg_Y(int index)
 {
-  Inst_Printf("ld", "[%s-%d],%%o0", asm_reg_e, (index + 4) * 4);
+  Inst_Printf("ld", "[%s%+d],%%o0", asm_reg_e, Y_OFFSET(index));
 }
 
 
@@ -360,7 +388,7 @@ Move_To_Reg_X(int index)
 void
 Move_To_Reg_Y(int index)
 {
-  Inst_Printf("st", "%%o0,[%s-%d]", asm_reg_e, (index + 4) * 4);
+  Inst_Printf("st", "%%o0,[%s%+d]", asm_reg_e, Y_OFFSET(index));
 }
 
 
@@ -380,21 +408,21 @@ Call_C_Start(char *fct_name, int fc, int nb_args, char **p_inline)
 
 #define MAX_ARGS_IN_REGS 6
 
-#define BEFORE_ARG                                                          \
-{                                                                           \
-  char r[4];                                                                \
-                                                                            \
-  if (offset < MAX_ARGS_IN_REGS)                                            \
-    sprintf(r, "%%o%d", offset);                                            \
-  else                                                                      \
+#define BEFORE_ARG				\
+{						\
+  char r[4];					\
+						\
+  if (offset < MAX_ARGS_IN_REGS)		\
+    sprintf(r, "%%o%d", offset);		\
+  else						\
     strcpy(r, "%l7");
 
 
-#define AFTER_ARG                                                           \
-  if (offset >= MAX_ARGS_IN_REGS)                                           \
-    Delay_Printf("st","%s,[%%sp+%d]", r,                                    \
-                 92 + (offset - MAX_ARGS_IN_REGS) * 4);                     \
- }
+#define AFTER_ARG					\
+  if (offset >= MAX_ARGS_IN_REGS)			\
+    Delay_Printf("st","%s,[%%sp+%d]", r,		\
+                 92 + (offset - MAX_ARGS_IN_REGS) * 4);	\
+}
 
 
 
@@ -530,9 +558,9 @@ Call_C_Arg_Reg_Y(int offset, int adr_of, int index)
   BEFORE_ARG;
 
   if (adr_of)
-    Delay_Printf("add", "%s,-%d,%s", asm_reg_e, (index + 4) * 4, r);
+    Delay_Printf("add", "%s,%+d,%s", asm_reg_e, Y_OFFSET(index), r);
   else
-    Delay_Printf("ld", "[%s-%d],%s", asm_reg_e, (index + 4) * 4, r);
+    Delay_Printf("ld", "[%s%+d],%s", asm_reg_e, Y_OFFSET(index), r);
 
   AFTER_ARG;
 
@@ -710,7 +738,7 @@ Move_Ret_To_Reg_X(int index)
 void
 Move_Ret_To_Reg_Y(int index)
 {				/* same as Move_To_Reg_Y */
-  Inst_Printf("st", "%%o0,[%s-%d]", asm_reg_e, (index + 4) * 4);
+  Inst_Printf("st", "%%o0,[%s%+d]", asm_reg_e, Y_OFFSET(index));
 }
 
 
