@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: code_gen.pl,v 1.11 2002/04/19 15:43:36 diaz Exp $ */
+/* $Id: code_gen.pl,v 1.12 2002/06/24 10:26:24 diaz Exp $ */
 
 code_generation(Head, Body, NbChunk, NbY, WamHead) :-
 	g_assign(last_pred, f),
@@ -594,6 +594,8 @@ math_exp_functor_name(>>, 2, 'Fct_Shr').
 math_exp_functor_name(abs, 1, 'Fct_Abs').
 math_exp_functor_name(sign, 1, 'Fct_Sign').
 
+math_exp_functor_name(min, 2, 'Fct_Min').
+math_exp_functor_name(max, 2, 'Fct_Max').
 math_exp_functor_name(**, 2, 'Fct_Pow').
 math_exp_functor_name(sqrt, 1, 'Fct_Sqrt').
 math_exp_functor_name(atan, 1, 'Fct_Atan').
