@@ -6,7 +6,7 @@
  * Descr.: main file                                                       *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: pl2wam.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: pl2wam.pl,v 1.7 2002/03/19 18:24:38 diaz Exp $ */
 
 pl2wam(LArg) :-
 	catch(pl2wam1(LArg), Err, exception(Err)).
@@ -246,7 +246,7 @@ display_counters :-
 
 
 
-          /*--- command-line options reading ---*/
+          % Command-line options reading
 
 cmd_line_args(LArg, PlFile, WamFile) :-
 	g_assign(plfile, ''),
@@ -400,7 +400,7 @@ cmd_line_arg1(PlFile, LArg, LArg) :-
 
 
 
-          /*--- copying ---*/
+          % Copying
 
 display_copying :-
 	prolog_name(Name),
@@ -433,7 +433,7 @@ prolog_copyright(Copyright) :-
 
 
 
-          /*--- help ---*/
+          % Help
 
 h('Usage: pl2wam [OPTION...] FILE').
 h('').
@@ -464,7 +464,7 @@ h('''user'' can be given as FILE for the standard input/output').
 
 
 
-          /*--- starting directive ---*/
+          % Starting directive
 
 go :-
 	argument_list(LArg),
