@@ -6,7 +6,7 @@
  * Descr.: code emission                                                   *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,24 +22,24 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: wam_emit.pl,v 1.5 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: wam_emit.pl,v 1.8 2002/04/12 14:36:26 diaz Exp $ */
 
 /*-------------------------------------------------------------------------*
  * WAM Instructions                                                        *
  *                                                                         *
- * get_variable(V,A)                        put_variable(V,A)              *
+ * get_variable(V, A)                       put_variable(V, A)             *
  *                                          put_void(A)                    *
- * get_value(V,A)                           put_value(V,A)                 *
- *                                          put_unsafe_value(y(Y),A)       *
- * get_atom(F,A)                            put_atom(F,A)                  *
- * get_integer(I,A)                         put_integer(N,A)               *
- * get_float(N,A)                           put_float(N,A)                 *
+ * get_value(V, A)                          put_value(V, A)                *
+ *                                          put_unsafe_value(y(Y), A)      *
+ * get_atom(F, A)                           put_atom(F, A)                 *
+ * get_integer(I, A)                        put_integer(N, A)              *
+ * get_float(D, A)                          put_float(D, A)                *
  * get_nil(A)                               put_nil(A)                     *
  * get_list(A)                              put_list(A)                    *
- * get_structure(F/N,A)                     put_structure(F/N,A)           *
+ * get_structure(F/N, A)                    put_structure(F/N, A)          *
  *                                                                         *
- *                                          math_load_value(V,A)           *
- *                                          math_fast_load_value(V,A)      *
+ *                                          math_load_value(V, A)          *
+ *                                          math_fast_load_value(V, A)     *
  *                                                                         *
  * unify_variable(V)                        allocate(N)                    *
  * unify_void(N)                            deallocate                     *
@@ -53,7 +53,7 @@
  *                                                                         *
  * label(L)                                                                *
  *                                                                         *
- * switch_on_term(Lvar,Latm,Lint,Llst,Lstc)                                *
+ * switch_on_term(Lvar, Latm, Lint, Llst, Lstc)                            *
  * switch_on_atom([(F,L),...])                                             *
  * switch_on_integer([(N,L),...])                                          *
  * switch_on_structure([(F/N,L),...])                                      *
@@ -65,20 +65,25 @@
  * load_cut_level(A)                                                       *
  * cut(V)                                                                  *
  *                                                                         *
- * function(F,X,[X,...])                                                   *
- * call_c(F,[X,...])                                                       *
- * call_c_test(F,[X1,...])                                                 *
- * call_c_jump(F,[X1,...])                                                 *
- * foreign_call_c(F,T0,F0,N0,K,[(T1,N1)...],[X1,...])                      *
- *                F=FctName, T0=ret type F0/N0=BipName/BipArity K=ChcSize  *
- *                Ti=type Ni=0:in 1:out 2:in/out                           *
+ * call_c(F, [T,...], [W,...])                                             *
+ *   F=FctName, T=option only these options are relevant:                  *
+ *    - jump/boolean/x(X) (jump at / test / move returned value)           *
+ *    - set_cp (set CP before the call at the next instruction)            *
+ *    - fast_call (use a fact call convention)                             *
+ *    - tagged (use tagged calls for atoms, integers and F/N)              *
  *                                                                         *
- * V  : x(X) or y(Y)                                                       *
- * X,Y: integer >= 0                                                       *
- * A  : integer                                                            *
- * N,K: integer                                                            *
- * F,T: atom                                                               *
- * L  : integer >= 1 (with no "holes") or 'fail' inside switch_on_term     *
+ * foreign_call_c(F, T0, P/N, K, [(M1, T1),...])                           *
+ *   F=FctName, T0=Return, P/N=BipName/BipArity, K=ChcSize                 *
+ *   Mi=mode (in/out/in_out), Ti=type                                      *
+ *                                                                         *
+ * V      : x(X) or y(Y)                                                   *
+ * X, Y   : integer >= 0                                                   *
+ * A      : integer                                                        *
+ * D      : float                                                          *
+ * N, K   : integer                                                        *
+ * F, T, M: atom                                                           *
+ * W      : atom or integer or float or atom/integer or x(X)               *
+ * L      : integer >= 1 (with no "holes") or 'fail' inside switch_on_term *
  *-------------------------------------------------------------------------*/
 
 emit_code_init(WamFile0, PlFile0) :-
@@ -193,8 +198,8 @@ emit_wam_code(WamInst, First, Stream) :-
 	emit_wam_code(WamInst1, First, Stream).
 
 emit_wam_code(WamInst, _, _) :-
-	g_read(keep_void_inst, f),
-	dummy_instruction(WamInst), !.
+	g_read(keep_void_inst, KeepVoidInst),
+	dummy_instruction(WamInst, KeepVoidInst), !.
 
 emit_wam_code(WamInst, First, Stream) :-
 	WamInst = label(_), !,
@@ -234,13 +239,36 @@ emit_args(N, N, _, _) :-
 emit_args(I, N, WamInst, Stream) :-
 	I1 is I + 1,
 	(   I1 = 1 ->
-	    Car = '('
-	;   Car = (',')
+	    put_char(Stream, '(')
+	;   put_char(Stream, ',')
 	),
 	arg(I1, WamInst, A),
-	format(Stream, '~a~q', [Car, A]),
+	emit_one_arg(A, Stream),
 	emit_args(I1, N, WamInst, Stream).
 
+emit_one_arg([X|L], Stream) :-
+	length(L, N),		% split long lists
+	N > 30,
+	!,
+	put_char(Stream, '['),
+	line_position(Stream, P),
+	write_term(Stream, X, [quoted(true), priority(999)]),
+	emit_list(L, P, Stream).
+
+emit_one_arg(A, Stream) :-
+	writeq(Stream, A).
+
+
+
+
+emit_list([], _, Stream) :-
+	put_char(Stream, ']').
+
+emit_list([X|L], P, Stream) :-
+	format(Stream, ',~n~*c', [P, 0' ]),
+	write_term(Stream, X, [quoted(true), priority(999)]),
+	emit_list(L, P, Stream).
+
 
 
 
