@@ -6,7 +6,7 @@
  * Descr.: Prolog hexadecimal decoding filter                              *
  * Author: Daniel Diaz                                                     *
  *                                                                         *
- * Copyright (C) 1999-2001 Daniel Diaz                                     *
+ * Copyright (C) 1999-2002 Daniel Diaz                                     *
  *                                                                         *
  * GNU Prolog is free software; you can redistribute it and/or modify it   *
  * under the terms of the GNU General Public License as published by the   *
@@ -22,7 +22,7 @@
  * 59 Temple Place - Suite 330, Boston, MA 02111, USA.                     *
  *-------------------------------------------------------------------------*/
 
-/* $Id: hexfilter.c,v 1.4 2001/09/17 16:01:51 diaz Exp $ */
+/* $Id: hexfilter.c,v 1.6 2002/03/19 18:24:38 diaz Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -84,7 +84,7 @@ void Display_Help(void);
 
 
 
-#define Check_Arg(i,str)           (strncmp(argv[i],str,strlen(argv[i]))==0)
+#define Check_Arg(i, str)       (strncmp(argv[i], str, strlen(argv[i])) == 0)
 
 
 
@@ -163,7 +163,7 @@ One_Line(char *str)
   int n;
   char *p;
 
-#define Is_Sep(c) (c=='\0' || isspace(c) || strchr("'\"",c))
+#define Is_Sep(c) (c == '\0' || isspace(c) || strchr("'\"", c))
 
   if (encode)
     {
@@ -188,7 +188,8 @@ One_Line(char *str)
 		      break;
 		    }
 		}
-	      printf("%2X", *str++);
+	      printf("%2X", (unsigned) (unsigned char) *str);
+	      str++;
 	    }
 	}
       return;
@@ -320,7 +321,7 @@ Fatal_Error(char *format, ...)
  *-------------------------------------------------------------------------*/
 void
 Display_Help(void)
-#define L(msg)  fprintf(stderr,"%s\n",msg)
+#define L(msg)  fprintf(stderr, "%s\n", msg)
 {
   fprintf(stderr, "Usage: %s [OPTION]... [FILE...]", HEXGPLC);
   L(" ");
